"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var core_1 = require("@angular/core");
var color_1 = require("tns-core-modules/color");
var platform = require("tns-core-modules/platform");
var view_1 = require("tns-core-modules/ui/core/view");
var weak_event_listener_1 = require("tns-core-modules/ui/core/weak-event-listener");
var gestures_1 = require("tns-core-modules/ui/gestures/gestures");
var style_properties_1 = require("tns-core-modules/ui/styling/style-properties");
var NativeRippleDirective = (function () {
    function NativeRippleDirective(el) {
        var _this = this;
        this.el = el;
        this.rippleColorAlpha = 0.25; // multiplicative with rippleColor and the default ripple opacity of 0.5 (from RippleDrawable)
        this.rippleLayer = "foreground";
        this.loaded = false;
        this.initialized = false;
        this.holding = false;
        this.monkeyPatch = function (val) {
            _this.previousNSFn.call(_this.el.nativeElement, val);
            _this.applyOrRemoveRipple();
        };
        if (platform.isAndroid) {
            this.originalNSFn = this.el.nativeElement._redrawNativeBackground; // always store the original method
        }
    }
    NativeRippleDirective.prototype.ngOnInit = function () {
        if (!this.initialized) {
            this.initialized = true;
            weak_event_listener_1.addWeakEventListener(this.el.nativeElement, "loaded", this.onLoaded, this);
            weak_event_listener_1.addWeakEventListener(this.el.nativeElement, "unloaded", this.onUnloaded, this);
            if (this.el.nativeElement.isLoaded) {
                this.onLoaded();
            }
        }
    };
    NativeRippleDirective.prototype.ngOnDestroy = function () {
        if (this.initialized) {
            this.initialized = false;
            weak_event_listener_1.removeWeakEventListener(this.el.nativeElement, "loaded", this.onLoaded, this);
            weak_event_listener_1.removeWeakEventListener(this.el.nativeElement, "unloaded", this.onUnloaded, this);
        }
    };
    NativeRippleDirective.prototype.ngOnChanges = function (changes) {
        if ((changes.ripple && changes.ripple.currentValue !== changes.ripple.previousValue) ||
            (changes.rippleColor && changes.rippleColor.currentValue !== changes.rippleColor.currentValue)) {
            this.applyOrRemoveRipple();
        }
    };
    NativeRippleDirective.prototype.getRippleColor = function () {
        var c = new color_1.Color(this.rippleColor || "#000000");
        c = new color_1.Color(c.a * (this.rippleColorAlpha || 0.25), c.r, c.g, c.b);
        return c;
    };
    NativeRippleDirective.prototype.getRippleLayer = function () {
        if (platform.isAndroid) {
            var MARSHMALLOW = 23;
            if (android.os.Build.VERSION.SDK_INT < MARSHMALLOW) {
                return "background";
            }
        }
        return (this.rippleLayer || "foreground");
    };
    NativeRippleDirective.prototype.applyOrRemoveRipple = function () {
        if (!this.loaded) {
            return;
        }
        if (this.ripple !== "off") {
            this.applyRipple();
        }
        else {
            this.removeRipple();
        }
    };
    // @HostListener('loaded')
    NativeRippleDirective.prototype.onLoaded = function () {
        this.loaded = true;
        if (!this.initialized) {
            this.ngOnInit();
        }
        this.applyOrRemoveRipple();
        // WARNING: monkey patching {N} functions ahead
        if (platform.isAndroid) {
            this.previousNSFn = this.el.nativeElement._redrawNativeBackground; // monkey should play nice with other monkeys
            this.el.nativeElement._redrawNativeBackground = this.monkeyPatch;
        }
    };
    // @HostListener('unloaded')
    NativeRippleDirective.prototype.onUnloaded = function () {
        this.loaded = false;
        this.removeRipple();
        // remove monkey patch
        if (platform.isAndroid) {
            this.el.nativeElement._redrawNativeBackground = this.originalNSFn; // always revert to the original
        }
    };
    NativeRippleDirective.prototype.getInDP = function (radius) {
        return style_properties_1.Length.toDevicePixels(radius, 0);
    };
    NativeRippleDirective.prototype.applyRipple = function () {
        if (platform.isAndroid) {
            this.applyOnAndroid();
        }
        else if (platform.isIOS) {
            this.applyOnIOS();
        }
    };
    NativeRippleDirective.prototype.removeRipple = function () {
        if (platform.isIOS) {
            this.removeOnIOS();
        }
        else if (platform.isAndroid) {
            this.removeOnAndroid();
        }
    };
    NativeRippleDirective.prototype.removeHold = function () {
        if (this.holdAnimation) {
            this.holdAnimation.removeFromSuperview();
            this.holdAnimation = null;
        }
    };
    NativeRippleDirective.prototype.performRipple = function (x, y) {
        if (!(this.el.nativeElement instanceof view_1.View)) {
            return;
        }
        this.removeHold();
        var nativeView = this.el.nativeElement.ios;
        var scale = 8.0;
        var size = this.el.nativeElement.getActualSize();
        x = Math.min(Math.max(x, 0), size.width);
        y = Math.min(Math.max(y, 0), size.height);
        var radius = 30;
        var ripple = UIView.alloc().initWithFrame(CGRectMake(0, 0, radius, radius));
        ripple.layer.cornerRadius = radius * 0.5;
        ripple.backgroundColor = this.getRippleColor().ios;
        ripple.alpha = NativeRippleDirective.IOS_RIPPLE_ALPHA;
        ripple.userInteractionEnabled = false;
        nativeView.insertSubviewAtIndex(ripple, 0);
        ripple.center = CGPointMake(x || 0, y || 0);
        UIView.animateWithDurationDelayOptionsAnimationsCompletion(0.4, 0, UIViewAnimationOptionCurveEaseOut, function () {
            ripple.transform = CGAffineTransformMakeScale(scale, scale);
            ripple.alpha = 0.0;
        }, function (finished) {
            ripple.removeFromSuperview();
        });
    };
    NativeRippleDirective.prototype.performHold = function () {
        if (!(this.el.nativeElement instanceof view_1.View)) {
            return;
        }
        if (this.holdAnimation) {
            this.removeHold(); // this should never happen, but...
        }
        var nativeView = this.el.nativeElement.ios;
        var scale = 8.0;
        var size = this.el.nativeElement.getActualSize();
        var holdanim = UIView.alloc().initWithFrame(CGRectMake(0, 0, size.width, size.height));
        holdanim.backgroundColor = this.getRippleColor().ios;
        holdanim.alpha = 0.0;
        holdanim.userInteractionEnabled = false;
        nativeView.insertSubviewAtIndex(holdanim, 0);
        holdanim.center = CGPointMake(size.width / 2.0, size.height / 2.0);
        this.holdAnimation = holdanim;
        UIView.animateWithDurationDelayOptionsAnimationsCompletion(0.6, 0, UIViewAnimationOptionCurveEaseOut, function () {
            // holdanim.transform = CGAffineTransformMakeScale(scale, scale);
            holdanim.alpha = NativeRippleDirective.IOS_RIPPLE_ALPHA;
            // holdanim.backgroundColor = new Color(
            //     this.rippleColor || '#400000'
            // ).ios;
        }, function (finished) {
            // holdanim.removeFromSuperview();
        });
    };
    NativeRippleDirective.prototype.applyOnIOS = function () {
        var _this = this;
        if (this.el.nativeElement instanceof view_1.View) {
            this.tapFn = function (args) {
                if (_this.holding && args.action === 'move' &&
                    (args.getX() < 0 || args.getX() > _this.el.nativeElement.getActualSize().width ||
                        args.getY() < 0 || args.getY() > _this.el.nativeElement.getActualSize().height)) {
                    _this.holding = false;
                    _this.performRipple(args.getX(), args.getY());
                }
                else if (args.action === 'down' && !_this.holding) {
                    _this.holding = true;
                    _this.performHold();
                }
                else if (_this.holding &&
                    (args.action === 'up' || args.action === 'cancel')) {
                    _this.holding = false;
                    _this.performRipple(args.getX(), args.getY());
                }
            };
            this.el.nativeElement.ios.clipsToBounds = true;
            this.el.nativeElement.on(gestures_1.GestureTypes.touch, this.tapFn);
        }
    };
    NativeRippleDirective.prototype.removeOnIOS = function () {
        if (this.el.nativeElement instanceof view_1.View) {
            this.removeHold();
            this.el.nativeElement.off(gestures_1.GestureTypes.touch, this.tapFn);
        }
    };
    NativeRippleDirective.prototype.applyOnAndroid = function () {
        if (this.el.nativeElement instanceof view_1.View && this.el.nativeElement.android) {
            var LOLLIPOP = 21;
            var MARSHMALLOW = 23;
            if (android.os.Build.VERSION.SDK_INT >= LOLLIPOP) {
                var androidView = this.el.nativeElement.android;
                if ((this.getRippleLayer() === "background" && android.os.Build.VERSION.SDK_INT >= MARSHMALLOW &&
                    androidView.getForeground() instanceof android.graphics.drawable.RippleDrawable) ||
                    (this.getRippleLayer() === "foreground" && androidView.getBackground() instanceof android.graphics.drawable.RippleDrawable)) {
                    this.removeOnAndroid(); // remove old ripples
                }
                var originalBg = this.getRippleLayer() === "background" ? androidView.getBackground() : androidView.getForeground();
                var mask = void 0;
                if (originalBg instanceof android.graphics.drawable.RippleDrawable && originalBg.getNumberOfLayers() < 2) {
                    // previous ripple didn't need a mask!
                    mask = null;
                }
                else {
                    var outerRadii = Array.create('float', 8);
                    outerRadii[0] = outerRadii[1] = this.getInDP(this.el.nativeElement.borderTopLeftRadius);
                    outerRadii[2] = outerRadii[3] = this.getInDP(this.el.nativeElement.borderTopRightRadius);
                    outerRadii[4] = outerRadii[5] = this.getInDP(this.el.nativeElement.borderBottomRightRadius);
                    outerRadii[6] = outerRadii[7] = this.getInDP(this.el.nativeElement.borderBottomLeftRadius);
                    var r = new android.graphics.drawable.shapes.RoundRectShape(outerRadii, null, null);
                    mask = new android.graphics.drawable.ShapeDrawable(r);
                    mask.getPaint().setColor(android.graphics.Color.BLACK);
                }
                var rippleBg = this.getRippleLayer() === "background" ? androidView.getBackground() : androidView.getForeground();
                if (rippleBg == null) {
                    rippleBg = new android.graphics.drawable.ColorDrawable(new color_1.Color("transparent").android);
                }
                var drawable = new android.graphics.drawable.RippleDrawable(android.content.res.ColorStateList.valueOf(this.getRippleColor().android), rippleBg instanceof android.graphics.drawable.RippleDrawable ?
                    rippleBg.getDrawable(0) : rippleBg, mask);
                if (this.getRippleLayer() === "background") {
                    androidView.setBackground(drawable);
                }
                else {
                    androidView.setForeground(drawable);
                }
            }
        }
    };
    NativeRippleDirective.prototype.removeOnAndroid = function () {
        if (this.el.nativeElement instanceof view_1.View && this.el.nativeElement.android) {
            var LOLLIPOP = 21;
            var MARSHMALLOW = 23;
            if (android.os.Build.VERSION.SDK_INT >= LOLLIPOP) {
                var androidView = this.el.nativeElement.android;
                if (android.os.Build.VERSION.SDK_INT >= MARSHMALLOW) {
                    if (androidView.getForeground() instanceof android.graphics.drawable.RippleDrawable) {
                        androidView.setForeground(androidView.getForeground().getDrawable(0));
                    }
                }
                if (androidView.getBackground() instanceof android.graphics.drawable.RippleDrawable) {
                    androidView.setBackground(androidView.getBackground().getDrawable(0));
                }
            }
        }
    };
    NativeRippleDirective.IOS_RIPPLE_ALPHA = 0.5;
    NativeRippleDirective.decorators = [
        { type: core_1.Directive, args: [{ selector: '[ripple]' },] },
    ];
    /** @nocollapse */
    NativeRippleDirective.ctorParameters = function () { return [
        { type: core_1.ElementRef, },
    ]; };
    NativeRippleDirective.propDecorators = {
        'ripple': [{ type: core_1.Input },],
        'rippleColor': [{ type: core_1.Input },],
        'rippleColorAlpha': [{ type: core_1.Input },],
        'rippleLayer': [{ type: core_1.Input },],
    };
    return NativeRippleDirective;
}());
exports.NativeRippleDirective = NativeRippleDirective;
