"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var color_1 = require("tns-core-modules/color");
var platform = require("tns-core-modules/platform");
var page_1 = require("tns-core-modules/ui/page/page");
var style_properties_1 = require("tns-core-modules/ui/styling/style-properties");
var ripple_common_1 = require("./ripple.common");
var MARSHMALLOW = 23;
var LOLLIPOP = 21;
function getInDP(radius) {
    return style_properties_1.Length.toDevicePixels(radius, 0);
}
var Ripple = /** @class */ (function (_super) {
    __extends(Ripple, _super);
    function Ripple(tnsView) {
        var _this = _super.call(this, tnsView) || this;
        _this.hasCachedBackground = true;
        _this._rippleLayer = android.os.Build.VERSION.SDK_INT < MARSHMALLOW ? "background" : "auto";
        _this._enabled = android.os.Build.VERSION.SDK_INT >= LOLLIPOP;
        _this.monkeyPatch = function (val) {
            var tnsView = _this.tnsView.get();
            if (tnsView) {
                if (_this.hasCachedBackground && tnsView.android.getBackground() !== _this.cachedViewBackground) {
                    tnsView.android.setBackground(_this.cachedViewBackground);
                }
                _this.previousNSFn.call(tnsView, val);
                _this.hasCachedBackground = true;
                _this.cachedViewBackground = tnsView.android.getBackground();
                _this.refreshRippleState();
            }
        };
        _this.originalNSFn = tnsView._redrawNativeBackground;
        return _this;
    }
    Object.defineProperty(Ripple.prototype, "enabled", {
        get: function () {
            return this._enabled;
        },
        set: function (v) {
            if (android.os.Build.VERSION.SDK_INT >= LOLLIPOP) {
                this._enabled = !!v;
                this.refreshRippleState();
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Ripple.prototype, "rippleLayer", {
        get: function () {
            return this._rippleLayer;
        },
        set: function (layer) {
            var oldLayer = this._rippleLayer;
            if (platform.isAndroid) {
                if (android.os.Build.VERSION.SDK_INT < MARSHMALLOW) {
                    if (this._rippleLayer !== "background") {
                        this._rippleLayer = "background";
                    }
                }
                else if (["background", "foreground", "auto"].indexOf(layer) >= 0 && layer !== this._rippleLayer) {
                    this._rippleLayer = layer;
                }
                else {
                    this._rippleLayer = "auto";
                }
            }
            if (oldLayer !== this._rippleLayer) {
                this.refreshRippleState();
            }
        },
        enumerable: true,
        configurable: true
    });
    Ripple.prototype.init = function () {
        var tnsView = this.tnsView.get();
        if (tnsView) {
            this.previousNSFn = tnsView._redrawNativeBackground; // monkey should play nice with other monkeys
            tnsView._redrawNativeBackground = this.monkeyPatch;
        }
        _super.prototype.init.call(this);
    };
    Ripple.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
        var tnsView = this.tnsView.get();
        if (tnsView) {
            tnsView._redrawNativeBackground = this.originalNSFn;
        }
        this.hasCachedBackground = false;
        this.cachedRippleDrawable = null;
        this.cachedViewBackground = null;
    };
    Ripple.prototype.onLoaded = function () {
        this.refreshRippleState();
    };
    Ripple.prototype.apply = function () {
        var tnsView = this.tnsView.get();
        if (tnsView && tnsView instanceof page_1.View && tnsView.android) {
            if (android.os.Build.VERSION.SDK_INT >= LOLLIPOP) {
                var androidView = tnsView.android;
                var actualRipple = this.getTargetRipple();
                var targetLayer = this.getTargetLayer();
                // remove conflicting ripples
                if (targetLayer === "foreground" && androidView.getBackground() instanceof android.graphics.drawable.RippleDrawable) {
                    androidView.setBackground(androidView.getBackground().getDrawable(0));
                }
                else if (targetLayer === "background" && android.os.Build.VERSION.SDK_INT >= MARSHMALLOW && androidView.getForeground() instanceof android.graphics.drawable.RippleDrawable) {
                    androidView.setForeground(androidView.getForeground().getDrawable(0));
                }
                var originalBg = targetLayer === "background" ? androidView.getBackground() : androidView.getForeground();
                if (!originalBg) {
                    originalBg = new android.graphics.drawable.ColorDrawable(new color_1.Color("transparent").android);
                }
                var drawable = void 0;
                var mask = void 0;
                if (originalBg instanceof android.graphics.drawable.RippleDrawable && originalBg.getNumberOfLayers() < 2) {
                    // previous ripple didn't need a mask!
                    mask = null;
                }
                else {
                    var outerRadii = Array.create('float', 8);
                    outerRadii[0] = outerRadii[1] = getInDP(tnsView.borderTopLeftRadius);
                    outerRadii[2] = outerRadii[3] = getInDP(tnsView.borderTopRightRadius);
                    outerRadii[4] = outerRadii[5] = getInDP(tnsView.borderBottomRightRadius);
                    outerRadii[6] = outerRadii[7] = getInDP(tnsView.borderBottomLeftRadius);
                    var r = new android.graphics.drawable.shapes.RoundRectShape(outerRadii, null, null);
                    mask = new android.graphics.drawable.ShapeDrawable(r);
                    mask.getPaint().setColor(android.graphics.Color.BLACK);
                }
                console.log(targetLayer, tnsView, this.rippleLayer);
                if (actualRipple) {
                    actualRipple.setColor(android.content.res.ColorStateList.valueOf(this.effectiveColor.android));
                    var layer0Id = actualRipple.getId(0);
                    actualRipple.setDrawableByLayerId(layer0Id, originalBg instanceof android.graphics.drawable.RippleDrawable ? originalBg.getDrawable(0) :
                        originalBg);
                    if (mask) {
                        var layer1Id = actualRipple.getId(1);
                        actualRipple.setDrawableByLayerId(layer1Id, mask);
                    }
                    drawable = actualRipple;
                }
                else {
                    drawable = new android.graphics.drawable.RippleDrawable(android.content.res.ColorStateList.valueOf(this.effectiveColor.android), originalBg instanceof android.graphics.drawable.RippleDrawable ? originalBg.getDrawable(0) :
                        originalBg, mask);
                    this.cachedRippleDrawable = drawable;
                }
                if (targetLayer === "background") {
                    androidView.setBackground(drawable);
                }
                else {
                    androidView.setForeground(drawable);
                }
                return true;
            }
        }
        return false;
    };
    Ripple.prototype.remove = function () {
        var tnsView = this.tnsView.get();
        if (tnsView && tnsView instanceof page_1.View && tnsView.android) {
            if (android.os.Build.VERSION.SDK_INT >= LOLLIPOP) {
                this.getTargetLayer();
                var androidView = tnsView.android;
                var targetLayer = this.getTargetLayer();
                if (targetLayer === "foreground" && androidView.getForeground() instanceof android.graphics.drawable.RippleDrawable) {
                    androidView.setForeground(androidView.getForeground().getDrawable(0));
                }
                else if (androidView.getBackground() instanceof android.graphics.drawable.RippleDrawable) {
                    androidView.setBackground(androidView.getBackground().getDrawable(0));
                }
                return true;
            }
        }
        return false;
    };
    Ripple.prototype.refreshRippleColor = function () {
        var actualRipple = this.getTargetRipple();
        if (actualRipple) {
            actualRipple.setColor(android.content.res.ColorStateList.valueOf(this.effectiveColor.android));
        }
    };
    Ripple.prototype.getTargetRipple = function () {
        var actualRipple;
        var tnsView = this.tnsView.get();
        if (tnsView) {
            var nativeView = tnsView.android;
            if (nativeView) {
                if (this._rippleLayer === "background" || this._rippleLayer === "auto") {
                    if (nativeView && nativeView.getBackground() instanceof android.graphics.drawable.RippleDrawable) {
                        actualRipple = nativeView.getBackground();
                    }
                }
                if (this._rippleLayer === "foreground" || (!actualRipple && this._rippleLayer === "auto")) {
                    if (nativeView && nativeView.getForeground() instanceof android.graphics.drawable.RippleDrawable) {
                        actualRipple = nativeView.getForeground();
                    }
                }
            }
            actualRipple = actualRipple || this.cachedRippleDrawable;
        }
        return actualRipple;
    };
    Ripple.prototype.getTargetLayer = function () {
        var target = this.rippleLayer === "auto" ? "foreground" : this.rippleLayer;
        var tnsView = this.tnsView.get();
        if (tnsView) {
            var nativeView = tnsView.android;
            if (this._rippleLayer === "auto") {
                if (nativeView) {
                    if (nativeView.getBackground() instanceof android.graphics.drawable.RippleDrawable) {
                        target = "background";
                    }
                    else if (nativeView.getForeground() instanceof android.graphics.drawable.RippleDrawable) {
                        target = "foreground";
                    }
                }
                else {
                    target = "foreground";
                }
            }
        }
        return target;
    };
    return Ripple;
}(ripple_common_1.RippleCommon));
exports.Ripple = Ripple;
