"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var color_1 = require("tns-core-modules/color");
var weak_event_listener_1 = require("tns-core-modules/ui/core/weak-event-listener/weak-event-listener");
var page_1 = require("tns-core-modules/ui/page/page");
var RippleCommon = /** @class */ (function () {
    function RippleCommon(tnsView) {
        this.effectiveColor = new color_1.Color(0.25 * 255, 0, 0, 0);
        this._enabled = true;
        this.rippleApplied = false;
        this.initialized = false;
        this.tnsView = new WeakRef(tnsView);
    }
    Object.defineProperty(RippleCommon.prototype, "enabled", {
        get: function () {
            return this._enabled;
        },
        set: function (v) {
            this._enabled = !!v;
            this.refreshRippleState();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RippleCommon.prototype, "rippleLayer", {
        get: function () {
            return this._rippleLayer;
        },
        set: function (layer) {
            /* */
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RippleCommon.prototype, "color", {
        get: function () {
            return this._rippleColor;
        },
        set: function (v) {
            this._rippleColor = v;
            this.invalidateColor();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(RippleCommon.prototype, "alpha", {
        get: function () {
            return this._rippleColorAlpha;
        },
        set: function (v) {
            this._rippleColorAlpha = v;
            this.invalidateColor();
        },
        enumerable: true,
        configurable: true
    });
    RippleCommon.prototype.init = function () {
        this.initialized = true;
        this.setEffectiveColor();
        this.bindEvents();
        this.refreshRippleState();
    };
    RippleCommon.prototype.dispose = function () {
        /* */
    };
    RippleCommon.prototype.refreshRippleState = function () {
        if (!this.initialized) {
            return;
        }
        if (this.enabled) {
            this.rippleApplied = this.apply();
        }
        else {
            this.rippleApplied = !this.remove();
        }
    };
    RippleCommon.prototype.apply = function () {
        return false;
    };
    RippleCommon.prototype.remove = function () {
        return false;
    };
    RippleCommon.prototype.invalidateColor = function () {
        this.setEffectiveColor();
        if (this.initialized) {
            this.refreshRippleColor();
        }
    };
    RippleCommon.prototype.refreshRippleColor = function () {
        /* */
    };
    RippleCommon.prototype.onLoaded = function () {
        /** */
    };
    RippleCommon.prototype.onUnloaded = function () {
        /** */
    };
    RippleCommon.prototype.setEffectiveColor = function () {
        var c;
        if (this._rippleColor) {
            if (this._rippleColor instanceof color_1.Color) {
                c = this._rippleColor;
            }
            else {
                c = new color_1.Color(this._rippleColor);
            }
        }
        else {
            c = new color_1.Color("#000000");
        }
        var alpha = RippleCommon.DEFAULT_RIPPLE_ALPHA;
        if (this.alpha !== null && this.alpha !== undefined) {
            alpha = +this.alpha;
        }
        c = new color_1.Color(c.a * alpha, c.r, c.g, c.b);
        this.effectiveColor = c;
    };
    RippleCommon.prototype.bindEvents = function () {
        var tnsView = this.tnsView.get();
        if (tnsView) {
            weak_event_listener_1.addWeakEventListener(tnsView, page_1.View.loadedEvent, this.onLoaded, this);
            weak_event_listener_1.addWeakEventListener(tnsView, page_1.View.unloadedEvent, this.onUnloaded, this);
            // in some cases, the element is already loaded by time of binding
            if (tnsView.isLoaded) {
                this.onLoaded();
            }
        }
    };
    RippleCommon.DEFAULT_RIPPLE_ALPHA = 0.25;
    return RippleCommon;
}());
exports.RippleCommon = RippleCommon;
