"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var gestures_1 = require("tns-core-modules/ui/gestures/gestures");
var page_1 = require("tns-core-modules/ui/page/page");
var ripple_common_1 = require("./ripple.common");
var Ripple = /** @class */ (function (_super) {
    __extends(Ripple, _super);
    function Ripple(tnsView) {
        var _this = _super.call(this, tnsView) || this;
        _this.holding = false;
        return _this;
    }
    Ripple.prototype.dispose = function () {
        _super.prototype.dispose.call(this);
        this.remove();
    };
    Ripple.prototype.removeHold = function () {
        if (this.holdAnimation) {
            this.holdAnimation.removeFromSuperview();
            this.holdAnimation = null;
        }
    };
    Ripple.prototype.performRipple = function (x, y) {
        var tnsView = this.tnsView.get();
        if (!(tnsView instanceof page_1.View)) {
            return;
        }
        this.removeHold();
        var nativeView = tnsView.ios;
        var scale = 8.0;
        var size = tnsView.getActualSize();
        x = Math.min(Math.max(x, 0), size.width);
        y = Math.min(Math.max(y, 0), size.height);
        var radius = 30;
        var ripple = UIView.alloc().initWithFrame(CGRectMake(0, 0, radius, radius));
        ripple.layer.cornerRadius = radius * 0.5;
        ripple.backgroundColor = this.effectiveColor.ios;
        ripple.alpha = Ripple.IOS_RIPPLE_ALPHA;
        ripple.userInteractionEnabled = false;
        nativeView.insertSubviewAtIndex(ripple, 0);
        ripple.center = CGPointMake(x || 0, y || 0);
        UIView.animateWithDurationDelayOptionsAnimationsCompletion(0.4, 0, UIViewAnimationOptionCurveEaseOut, function () {
            ripple.transform = CGAffineTransformMakeScale(scale, scale);
            ripple.alpha = 0.0;
        }, function (finished) {
            ripple.removeFromSuperview();
        });
    };
    Ripple.prototype.performHold = function () {
        var tnsView = this.tnsView.get();
        if (!(tnsView instanceof page_1.View)) {
            return;
        }
        if (this.holdAnimation) {
            this.removeHold(); // this should never happen, but...
        }
        var nativeView = tnsView.ios;
        var scale = 8.0;
        var size = tnsView.getActualSize();
        var holdanim = UIView.alloc().initWithFrame(CGRectMake(0, 0, size.width, size.height));
        holdanim.backgroundColor = this.effectiveColor.ios;
        holdanim.alpha = 0.0;
        holdanim.userInteractionEnabled = false;
        nativeView.insertSubviewAtIndex(holdanim, 0);
        holdanim.center = CGPointMake(size.width / 2.0, size.height / 2.0);
        this.holdAnimation = holdanim;
        UIView.animateWithDurationDelayOptionsAnimationsCompletion(0.6, 0, UIViewAnimationOptionCurveEaseOut, function () {
            holdanim.alpha = Ripple.IOS_RIPPLE_ALPHA;
        }, function (finished) {
            // holdanim.removeFromSuperview();
        });
    };
    Ripple.prototype.apply = function () {
        var _this = this;
        if (this.rippleApplied) {
            return true;
        }
        var outTnsView = this.tnsView.get();
        if (outTnsView instanceof page_1.View) {
            this.tapFn = function (args) {
                var tnsView = _this.tnsView.get();
                if (tnsView) {
                    if (_this.holding && args.action === 'move' &&
                        (args.getX() < 0 || args.getX() > tnsView.getActualSize().width ||
                            args.getY() < 0 || args.getY() > tnsView.getActualSize().height)) {
                        _this.holding = false;
                        _this.performRipple(args.getX(), args.getY());
                    }
                    else if (args.action === 'down' && !_this.holding) {
                        _this.holding = true;
                        _this.performHold();
                    }
                    else if (_this.holding &&
                        (args.action === 'up' || args.action === 'cancel')) {
                        _this.holding = false;
                        _this.performRipple(args.getX(), args.getY());
                    }
                }
            };
            outTnsView.ios.clipsToBounds = true;
            outTnsView.on(gestures_1.GestureTypes.touch, this.tapFn);
            return true;
        }
        return false;
    };
    Ripple.prototype.remove = function () {
        if (!this.rippleApplied) {
            return true;
        }
        var tnsView = this.tnsView.get();
        if (tnsView instanceof page_1.View) {
            this.removeHold();
            tnsView.off(gestures_1.GestureTypes.touch, this.tapFn);
            return true;
        }
        return false;
    };
    Ripple.IOS_RIPPLE_ALPHA = 0.5;
    return Ripple;
}(ripple_common_1.RippleCommon));
exports.Ripple = Ripple;
